import "dart:mirrors";
import "bindings/internal_bindings.dart";
import "xerum_binding.dart";
import "xerum_annotations.dart";
import "xerum_exceptions.dart";

class XerumContainer {
  var _bindings = new List<ResolvableXerumBinding>();

  bool canResolve(Type type) => _bindings.any((b) => b.contractType == type);

  dynamic resolve(Type type) {
    if (!canResolve(type)) {
      throw new XerumResolveException("Don't know how to resolve $type");
    }
    return _bindings.firstWhere((b) => b.contractType == type).resolve();
  }

  void register(XerumBinding binding,
      {bool asSingleton: false, bool replace: false}) {
    var contract = binding.contractType;
    var concrete = binding.concreteType;

    var contractMirror = reflectClass(contract);
    var concreteMirror = reflectClass(concrete);

    if (!concreteMirror.isAssignableTo(contractMirror)) {
      throw new XerumRegisterException(
          "$concrete does not implement $contract");
    }

    if (canResolve(contract)) {
      if (replace) {
        _bindings.removeWhere((b) => b.contractType == contract);
      } else {
        throw new XerumRegisterException("$contract is already registered");
      }
    }

    List<MethodMirror> constructors = new List.from(concreteMirror
        .declarations.values
        .where((c) => c is MethodMirror && c.isConstructor)
        .map((c) => c as MethodMirror));

    MethodMirror foundConstructor = _getAnnotatedConstructor(constructors);
    if (foundConstructor == null) {
      foundConstructor = _findUsableConstructor(constructors);
    }

    if (foundConstructor == null) {
      throw new XerumRegisterException(
          "No constructor found for $concrete that can be used with the currently registered services.");
    }

    var constructorArgs =
        foundConstructor.parameters.map((p) => resolve(p.type.reflectedType));
    var factoryFunction = () {
      return concreteMirror
          .newInstance(foundConstructor.constructorName, constructorArgs)
          .reflectee;
    };
    var newBinding = asSingleton
        ? new SingletonXerumBinding(binding, factoryFunction())
        : new TransientXerumBinding(binding, factoryFunction);
    _bindings.add(newBinding);
  }

  MethodMirror _getAnnotatedConstructor(List<MethodMirror> constructors) {
    for (var constructor in constructors) {
      for (var metadata in constructor.metadata) {
        if (metadata.hasReflectee &&
            metadata.reflectee.runtimeType == XerumInject) {
          return constructor;
        }
      }
    }
    return null;
  }

  MethodMirror _findUsableConstructor(List<MethodMirror> constructors) {
    for (var constructor in constructors) {
      if (constructor.parameters
          .every((p) => canResolve(p.type.reflectedType))) {
        return constructor;
      }
    }
    return null;
  }
}
